package cn.com.infosec.jce.provider;

import cn.com.infosec.asn1.ASN1EncodableVector;
import cn.com.infosec.asn1.ASN1InputStream;
import cn.com.infosec.asn1.ASN1Object;
import cn.com.infosec.asn1.ASN1Sequence;
import cn.com.infosec.asn1.ASN1TaggedObject;
import cn.com.infosec.asn1.DEREncodable;
import cn.com.infosec.asn1.DERInteger;
import cn.com.infosec.asn1.DERObject;
import cn.com.infosec.asn1.DERObjectIdentifier;
import cn.com.infosec.asn1.DERSequence;
import cn.com.infosec.asn1.x509.BasicConstraints;
import cn.com.infosec.asn1.x509.CRLDistPoint;
import cn.com.infosec.asn1.x509.DistributionPoint;
import cn.com.infosec.asn1.x509.DistributionPointName;
import cn.com.infosec.asn1.x509.GeneralName;
import cn.com.infosec.asn1.x509.GeneralNames;
import cn.com.infosec.asn1.x509.GeneralSubtree;
import cn.com.infosec.asn1.x509.IssuingDistributionPoint;
import cn.com.infosec.asn1.x509.NameConstraints;
import cn.com.infosec.asn1.x509.PolicyInformation;
import cn.com.infosec.asn1.x509.ReasonFlags;
import cn.com.infosec.asn1.x509.X509Extensions;
import cn.com.infosec.asn1.x509.X509Name;
import cn.com.infosec.jce.exception.ExtCertPathValidatorException;
import cn.com.infosec.util.Arrays;
import cn.com.infosec.x509.ExtendedPKIXBuilderParameters;
import cn.com.infosec.x509.ExtendedPKIXParameters;
import cn.com.infosec.x509.X509CRLStoreSelector;
import cn.com.infosec.x509.X509CertStoreSelector;
import java.io.IOException;
import java.math.BigInteger;
import java.security.GeneralSecurityException;
import java.security.PublicKey;
import java.security.cert.CertPath;
import java.security.cert.CertPathBuilder;
import java.security.cert.CertPathBuilderException;
import java.security.cert.CertPathValidatorException;
import java.security.cert.CertificateExpiredException;
import java.security.cert.CertificateNotYetValidException;
import java.security.cert.PKIXCertPathChecker;
import java.security.cert.X509CRL;
import java.security.cert.X509Certificate;
import java.security.cert.X509Extension;
import java.util.ArrayList;
import java.util.Collection;
import java.util.Date;
import java.util.Enumeration;
import java.util.HashMap;
import java.util.HashSet;
import java.util.Iterator;
import java.util.List;
import java.util.Set;
import javax.security.auth.x500.X500Principal;

public class RFC3280CertPathUtilities {
   protected static final String CERTIFICATE_POLICIES;
   protected static final String POLICY_MAPPINGS;
   protected static final String INHIBIT_ANY_POLICY;
   protected static final String ISSUING_DISTRIBUTION_POINT;
   protected static final String FRESHEST_CRL;
   protected static final String DELTA_CRL_INDICATOR;
   protected static final String POLICY_CONSTRAINTS;
   protected static final String BASIC_CONSTRAINTS;
   protected static final String CRL_DISTRIBUTION_POINTS;
   protected static final String SUBJECT_ALTERNATIVE_NAME;
   protected static final String NAME_CONSTRAINTS;
   protected static final String AUTHORITY_KEY_IDENTIFIER;
   protected static final String KEY_USAGE;
   protected static final String CRL_NUMBER;
   protected static final String ANY_POLICY = "2.5.29.32.0";
   protected static final int KEY_CERT_SIGN = 5;
   protected static final int CRL_SIGN = 6;
   protected static final String[] crlReasons;

   static {
      CERTIFICATE_POLICIES = X509Extensions.CertificatePolicies.getId();
      POLICY_MAPPINGS = X509Extensions.PolicyMappings.getId();
      INHIBIT_ANY_POLICY = X509Extensions.InhibitAnyPolicy.getId();
      ISSUING_DISTRIBUTION_POINT = X509Extensions.IssuingDistributionPoint.getId();
      FRESHEST_CRL = X509Extensions.FreshestCRL.getId();
      DELTA_CRL_INDICATOR = X509Extensions.DeltaCRLIndicator.getId();
      POLICY_CONSTRAINTS = X509Extensions.PolicyConstraints.getId();
      BASIC_CONSTRAINTS = X509Extensions.BasicConstraints.getId();
      CRL_DISTRIBUTION_POINTS = X509Extensions.CRLDistributionPoints.getId();
      SUBJECT_ALTERNATIVE_NAME = X509Extensions.SubjectAlternativeName.getId();
      NAME_CONSTRAINTS = X509Extensions.NameConstraints.getId();
      AUTHORITY_KEY_IDENTIFIER = X509Extensions.AuthorityKeyIdentifier.getId();
      KEY_USAGE = X509Extensions.KeyUsage.getId();
      CRL_NUMBER = X509Extensions.CRLNumber.getId();
      crlReasons = new String[]{"unspecified", "keyCompromise", "cACompromise", "affiliationChanged", "superseded", "cessationOfOperation", "certificateHold", "unknown", "removeFromCRL", "privilegeWithdrawn", "aACompromise"};
   }

   protected static void processCRLB2(DistributionPoint dp, Object cert, X509CRL crl) {
      IssuingDistributionPoint var10000;
      try {
         var10000 = IssuingDistributionPoint.getInstance(CertPathValidatorUtilities.getExtensionValue(crl, ISSUING_DISTRIBUTION_POINT));
      } catch (Exception var11) {
         throw new AnnotatedException("Issuing distribution point extension could not be decoded.", var11);
      }

      IssuingDistributionPoint var3 = var10000;
      if (var10000 != null) {
         if (var3.getDistributionPoint() != null) {
            DistributionPointName var4;
            DistributionPointName var25 = var4 = IssuingDistributionPoint.getInstance(var3).getDistributionPoint();
            ArrayList var5;
            var5 = new ArrayList.<init>();
            GeneralName[] var6;
            if (var25.getType() == 0) {
               var6 = GeneralNames.getInstance(var4.getName()).getNames();

               for(int var7 = 0; var7 < var6.length; ++var7) {
                  var5.add(var6[var7]);
               }
            }

            if (var4.getType() == 1) {
               label208: {
                  X509CRL var26 = crl;
                  ASN1EncodableVector crl1;
                  crl1 = new ASN1EncodableVector.<init>();

                  IOException var27;
                  label189: {
                     boolean var10001;
                     Enumeration var28;
                     try {
                        var28 = ASN1Sequence.getInstance(ASN1Object.fromByteArray(CertPathValidatorUtilities.getIssuerPrincipal(var26).getEncoded())).getObjects();
                     } catch (IOException var14) {
                        var27 = var14;
                        var10001 = false;
                        break label189;
                     }

                     Enumeration var21 = var28;

                     while(true) {
                        boolean var29;
                        try {
                           var29 = var21.hasMoreElements();
                        } catch (IOException var13) {
                           var27 = var13;
                           var10001 = false;
                           break;
                        }

                        if (!var29) {
                           crl1.add(var4.getName());
                           var5.add(new GeneralName(X509Name.getInstance(new DERSequence(crl1))));
                           break label208;
                        }

                        try {
                           crl1.add((DEREncodable)var21.nextElement());
                        } catch (IOException var12) {
                           var27 = var12;
                           var10001 = false;
                           break;
                        }
                     }
                  }

                  IOException dp1 = var27;
                  throw new AnnotatedException("Could not read CRL issuer.", dp1);
               }
            }

            boolean crl2 = false;
            GeneralName[] dp2;
            int var20;
            if (dp.getDistributionPoint() != null) {
               var25 = var4 = dp.getDistributionPoint();
               var6 = null;
               if (var25.getType() == 0) {
                  var6 = GeneralNames.getInstance(var4.getName()).getNames();
               }

               if (var4.getType() != 1) {
                  dp2 = var6;
               } else {
                  if (dp.getCRLIssuer() != null) {
                     dp2 = dp.getCRLIssuer().getNames();
                  } else {
                     GeneralName[] var32 = dp2 = new GeneralName[1];
                     byte var30 = 0;

                     try {
                        var32[var30] = new GeneralName(new X509Name((ASN1Sequence)ASN1Object.fromByteArray(CertPathValidatorUtilities.getEncodedIssuerPrincipal(cert).getEncoded())));
                     } catch (IOException var10) {
                        throw new AnnotatedException("Could not read certificate issuer.", var10);
                     }
                  }

                  for(int var22 = 0; var22 < dp2.length; ++var22) {
                     Enumeration var23 = ASN1Sequence.getInstance(dp2[var22].getName().getDERObject()).getObjects();
                     ASN1EncodableVector var8;
                     var8 = new ASN1EncodableVector.<init>();

                     while(var23.hasMoreElements()) {
                        var8.add((DEREncodable)var23.nextElement());
                     }

                     var8.add(var4.getName());
                     GeneralName var24;
                     var24 = new GeneralName.<init>(new X509Name(new DERSequence(var8)));
                     dp2[var22] = var24;
                  }
               }

               if (dp2 != null) {
                  for(var20 = 0; var20 < dp2.length; ++var20) {
                     if (var5.contains(dp2[var20])) {
                        crl2 = true;
                        break;
                     }
                  }
               }

               if (!crl2) {
                  throw new AnnotatedException("No match for certificate CRL issuing distribution point name to cRLIssuer CRL distribution point.");
               }
            } else {
               if (dp.getCRLIssuer() == null) {
                  throw new AnnotatedException("Either the cRLIssuer or the distributionPoint field must be contained in DistributionPoint.");
               }

               dp2 = dp.getCRLIssuer().getNames();

               for(var20 = 0; var20 < dp2.length; ++var20) {
                  if (var5.contains(dp2[var20])) {
                     crl2 = true;
                     break;
                  }
               }

               if (!crl2) {
                  throw new AnnotatedException("No match for certificate CRL issuing distribution point name to cRLIssuer CRL distribution point.");
               }
            }
         }

         BasicConstraints var31;
         Object var33;
         try {
            var33 = cert;
            var31 = BasicConstraints.getInstance(CertPathValidatorUtilities.getExtensionValue((X509Extension)cert, BASIC_CONSTRAINTS));
         } catch (Exception var9) {
            throw new AnnotatedException("Basic constraints extension could not be decoded.", var9);
         }

         BasicConstraints dp3 = var31;
         if (var33 instanceof X509Certificate) {
            if (var3.onlyContainsUserCerts() && dp3 != null && dp3.isCA()) {
               throw new AnnotatedException("CA Cert CRL only contains user certificates.");
            }

            if (var3.onlyContainsCACerts() && (dp3 == null || !dp3.isCA())) {
               throw new AnnotatedException("End CRL only contains CA certificates.");
            }
         }

         if (var3.onlyContainsAttributeCerts()) {
            throw new AnnotatedException("onlyContainsAttributeCerts boolean is asserted.");
         }
      }

   }

   protected static void processCRLB1(DistributionPoint dp, Object cert, X509CRL crl) {
      DERObject var3;
      DERObject var10000 = var3 = CertPathValidatorUtilities.getExtensionValue(crl, ISSUING_DISTRIBUTION_POINT);
      boolean var4 = false;
      if (var10000 != null && IssuingDistributionPoint.getInstance(var3).isIndirectCRL()) {
         var4 = true;
      }

      byte[] var9 = CertPathValidatorUtilities.getIssuerPrincipal(crl).getEncoded();
      boolean var5 = false;
      if (dp.getCRLIssuer() != null) {
         GeneralName[] dp1 = dp.getCRLIssuer().getNames();

         for(int cert1 = 0; cert1 < dp1.length; ++cert1) {
            if (dp1[cert1].getTagNo() == 4) {
               boolean var10;
               try {
                  var10 = Arrays.areEqual(dp1[cert1].getName().getDERObject().getEncoded(), var9);
               } catch (IOException var6) {
                  throw new AnnotatedException("CRL issuer information from distribution point cannot be decoded.", var6);
               }

               if (var10) {
                  var5 = true;
               }
            }
         }

         if (var5 && !var4) {
            throw new AnnotatedException("Distribution point contains cRLIssuer field but CRL is not indirect.");
         }

         if (!var5) {
            throw new AnnotatedException("CRL issuer of CRL does not match CRL issuer of distribution point.");
         }
      } else if (CertPathValidatorUtilities.getIssuerPrincipal(crl).equals(CertPathValidatorUtilities.getEncodedIssuerPrincipal(cert))) {
         var5 = true;
      }

      if (!var5) {
         throw new AnnotatedException("Cannot find matching CRL issuer for certificate.");
      }
   }

   protected static ReasonsMask processCRLD(X509CRL crl, DistributionPoint dp) {
      IssuingDistributionPoint var10000;
      try {
         var10000 = IssuingDistributionPoint.getInstance(CertPathValidatorUtilities.getExtensionValue(crl, ISSUING_DISTRIBUTION_POINT));
      } catch (Exception var3) {
         throw new AnnotatedException("Issuing distribution point extension could not be decoded.", var3);
      }

      IssuingDistributionPoint crl1 = var10000;
      if (var10000 != null && crl1.getOnlySomeReasons() != null && dp.getReasons() != null) {
         return (new ReasonsMask(dp.getReasons().intValue())).intersect(new ReasonsMask(crl1.getOnlySomeReasons().intValue()));
      } else if ((crl1 == null || crl1.getOnlySomeReasons() == null) && dp.getReasons() == null) {
         return ReasonsMask.allReasons;
      } else {
         ReasonsMask var2;
         ReasonsMask dp1;
         if (dp.getReasons() == null) {
            dp1 = ReasonsMask.allReasons;
            var2 = dp1;
         } else {
            var2 = new ReasonsMask.<init>(dp.getReasons().intValue());
         }

         if (crl1 == null) {
            ReasonsMask crl2 = ReasonsMask.allReasons;
            dp1 = crl2;
         } else {
            dp1 = new ReasonsMask.<init>(crl1.getOnlySomeReasons().intValue());
         }

         return var2.intersect(dp1);
      }
   }

   protected static Set processCRLF(X509CRL crl, Object cert, X509Certificate defaultCRLSignCert, PublicKey defaultCRLSignKey, ExtendedPKIXParameters paramsPKIX, List certPathCerts) {
      X509CertStoreSelector cert1;
      X509CertStoreSelector var10000 = cert1 = new X509CertStoreSelector;
      ExtendedPKIXParameters var10001 = paramsPKIX;
      X509CertStoreSelector var10002 = cert1;
      X509CRL var10003 = crl;
      cert1.<init>();

      try {
         var10002.setSubject(CertPathValidatorUtilities.getIssuerPrincipal(var10003).getEncoded());
      } catch (IOException var10) {
         throw new AnnotatedException("Subject criteria for certificate selector to find issuer certificate for CRL could not be set.", var10);
      }

      AnnotatedException crl2;
      Collection var51;
      boolean var52;
      X509Certificate var54;
      label176: {
         AnnotatedException var50;
         label180: {
            try {
               var51 = CertPathValidatorUtilities.findCertificates(var10000, var10001.getStores());
            } catch (AnnotatedException var37) {
               var50 = var37;
               var52 = false;
               break label180;
            }

            Collection crl1 = var51;

            X509CertStoreSelector var10004;
            ExtendedPKIXParameters var10005;
            Collection var59;
            try {
               var54 = defaultCRLSignCert;
               var59 = crl1;
               var10004 = cert1;
               var10005 = paramsPKIX;
               crl1.addAll(CertPathValidatorUtilities.findCertificates(cert1, paramsPKIX.getAdditionalStores()));
            } catch (AnnotatedException var36) {
               var50 = var36;
               var52 = false;
               break label180;
            }

            try {
               var59.addAll(CertPathValidatorUtilities.findCertificates(var10004, var10005.getCertStores()));
               break label176;
            } catch (AnnotatedException var35) {
               var50 = var35;
               var52 = false;
            }
         }

         crl2 = var50;
         throw new AnnotatedException("Issuer certificate for CRL cannot be searched.", crl2);
      }

      var51.add(var54);
      Iterator crl3 = var51.iterator();
      ArrayList cert2;
      cert2 = new ArrayList.<init>();
      ArrayList var6;
      var6 = new ArrayList.<init>();

      while(true) {
         while(crl3.hasNext()) {
            X509Certificate var7;
            if ((var7 = (X509Certificate)crl3.next()).equals(defaultCRLSignCert)) {
               cert2.add(var7);
               var6.add(defaultCRLSignKey);
            } else {
               CertPathBuilderException var57;
               label157: {
                  CertPathValidatorException var55;
                  label156: {
                     Exception var53;
                     label184: {
                        ExtendedPKIXParameters var56;
                        List var58;
                        X509Certificate var60;
                        CertPathBuilder var62;
                        try {
                           var58 = certPathCerts;
                           var60 = var7;
                           var56 = paramsPKIX;
                           var62 = CertPathBuilder.getInstance("PKIX", "INFOSEC");
                        } catch (CertPathBuilderException var32) {
                           var57 = var32;
                           var52 = false;
                           break label157;
                        } catch (CertPathValidatorException var33) {
                           var55 = var33;
                           var52 = false;
                           break label156;
                        } catch (Exception var34) {
                           var53 = var34;
                           var52 = false;
                           break label184;
                        }

                        CertPathBuilder var8 = var62;

                        X509CertStoreSelector var63;
                        try {
                           var63 = new X509CertStoreSelector;
                        } catch (CertPathBuilderException var29) {
                           var57 = var29;
                           var52 = false;
                           break label157;
                        } catch (CertPathValidatorException var30) {
                           var55 = var30;
                           var52 = false;
                           break label156;
                        } catch (Exception var31) {
                           var53 = var31;
                           var52 = false;
                           break label184;
                        }

                        X509CertStoreSelector var9 = var63;

                        ExtendedPKIXBuilderParameters var61;
                        try {
                           var9.<init>();
                           var63.setCertificate(var7);
                           var56 = (ExtendedPKIXParameters)var56.clone();
                           var56.setTargetCertConstraints(var9);
                           var61 = (ExtendedPKIXBuilderParameters)ExtendedPKIXBuilderParameters.getInstance(var56);
                        } catch (CertPathBuilderException var26) {
                           var57 = var26;
                           var52 = false;
                           break label157;
                        } catch (CertPathValidatorException var27) {
                           var55 = var27;
                           var52 = false;
                           break label156;
                        } catch (Exception var28) {
                           var53 = var28;
                           var52 = false;
                           break label184;
                        }

                        ExtendedPKIXBuilderParameters var49 = var61;

                        boolean var64;
                        try {
                           var64 = var58.contains(var60);
                        } catch (CertPathBuilderException var23) {
                           var57 = var23;
                           var52 = false;
                           break label157;
                        } catch (CertPathValidatorException var24) {
                           var55 = var24;
                           var52 = false;
                           break label156;
                        } catch (Exception var25) {
                           var53 = var25;
                           var52 = false;
                           break label184;
                        }

                        if (var64) {
                           try {
                              var49.setRevocationEnabled(false);
                           } catch (CertPathBuilderException var20) {
                              var57 = var20;
                              var52 = false;
                              break label157;
                           } catch (CertPathValidatorException var21) {
                              var55 = var21;
                              var52 = false;
                              break label156;
                           } catch (Exception var22) {
                              var53 = var22;
                              var52 = false;
                              break label184;
                           }
                        } else {
                           try {
                              var49.setRevocationEnabled(true);
                           } catch (CertPathBuilderException var17) {
                              var57 = var17;
                              var52 = false;
                              break label157;
                           } catch (CertPathValidatorException var18) {
                              var55 = var18;
                              var52 = false;
                              break label156;
                           } catch (Exception var19) {
                              var53 = var19;
                              var52 = false;
                              break label184;
                           }
                        }

                        ArrayList var65;
                        List var66;
                        try {
                           var65 = var6;
                           var66 = var8.build(var49).getCertPath().getCertificates();
                           cert2.add(var7);
                        } catch (CertPathBuilderException var14) {
                           var57 = var14;
                           var52 = false;
                           break label157;
                        } catch (CertPathValidatorException var15) {
                           var55 = var15;
                           var52 = false;
                           break label156;
                        } catch (Exception var16) {
                           var53 = var16;
                           var52 = false;
                           break label184;
                        }

                        try {
                           var65.add(CertPathValidatorUtilities.getNextWorkingKey(var66, 0));
                           continue;
                        } catch (CertPathBuilderException var11) {
                           var57 = var11;
                           var52 = false;
                           break label157;
                        } catch (CertPathValidatorException var12) {
                           var55 = var12;
                           var52 = false;
                           break label156;
                        } catch (Exception var13) {
                           var53 = var13;
                           var52 = false;
                        }
                     }

                     Exception crl4 = var53;
                     throw new RuntimeException(crl4.getMessage());
                  }

                  CertPathValidatorException crl5 = var55;
                  throw new AnnotatedException("Public key of issuer certificate of CRL could not be retrieved.", crl5);
               }

               CertPathBuilderException crl6 = var57;
               throw new AnnotatedException("Internal error.", crl6);
            }
         }

         HashSet var48;
         var48 = new HashSet.<init>();
         crl2 = null;

         for(int defaultCRLSignCert1 = 0; defaultCRLSignCert1 < cert2.size(); ++defaultCRLSignCert1) {
            boolean[] defaultCRLSignKey1;
            if ((defaultCRLSignKey1 = ((X509Certificate)cert2.get(defaultCRLSignCert1)).getKeyUsage()) == null || defaultCRLSignKey1.length >= 7 && defaultCRLSignKey1[6]) {
               var48.add(var6.get(defaultCRLSignCert1));
            } else {
               crl2 = new AnnotatedException.<init>("Issuer certificate key usage extension does not permit CRL signing.");
            }
         }

         if (var48.isEmpty() && crl2 == null) {
            throw new AnnotatedException("Cannot find a valid issuer certificate.");
         }

         if (var48.isEmpty() && crl2 != null) {
            throw crl2;
         }

         return var48;
      }
   }

   protected static PublicKey processCRLG(X509CRL crl, Set keys) {
      Set var10000 = keys;
      Exception keys1 = null;
      Iterator var2 = var10000.iterator();

      while(var2.hasNext()) {
         PublicKey var4 = (PublicKey)var2.next();
         PublicKey var10001 = var4;

         try {
            crl.verify(var10001);
            return var4;
         } catch (Exception var3) {
            keys1 = var3;
         }
      }

      throw new AnnotatedException("Cannot verify CRL.", keys1);
   }

   protected static X509CRL processCRLH(Set deltacrls, PublicKey key) {
      Set var10000 = deltacrls;
      Exception deltacrls1 = null;

      label37: {
         Exception var7;
         label41: {
            Iterator var8;
            boolean var10001;
            try {
               var8 = var10000.iterator();
            } catch (Exception var5) {
               var7 = var5;
               var10001 = false;
               break label41;
            }

            Iterator var2 = var8;

            boolean var9;
            try {
               var9 = var8.hasNext();
            } catch (Exception var4) {
               var7 = var4;
               var10001 = false;
               break label41;
            }

            if (!var9) {
               break label37;
            }

            try {
               X509CRL var10 = (X509CRL)var2.next();
               var10.verify(key);
               return var10;
            } catch (Exception var3) {
               var7 = var3;
               var10001 = false;
            }
         }

         deltacrls1 = var7;
      }

      if (deltacrls1 == null) {
         return null;
      } else {
         throw new AnnotatedException("Cannot verify delta CRL.", deltacrls1);
      }
   }

   protected static Set processCRLA1i(Date currentDate, ExtendedPKIXParameters paramsPKIX, X509Certificate cert, X509CRL crl) {
      HashSet var4;
      var4 = new HashSet.<init>();
      if (paramsPKIX.isUseDeltasEnabled()) {
         String cert1;
         CRLDistPoint var15;
         label47: {
            AnnotatedException var10000;
            label52: {
               X509Certificate var13;
               String var14;
               boolean var10001;
               try {
                  var13 = cert;
                  var14 = FRESHEST_CRL;
               } catch (AnnotatedException var10) {
                  var10000 = var10;
                  var10001 = false;
                  break label52;
               }

               cert1 = var14;

               try {
                  var15 = CRLDistPoint.getInstance(CertPathValidatorUtilities.getExtensionValue(var13, var14));
                  break label47;
               } catch (AnnotatedException var9) {
                  var10000 = var9;
                  var10001 = false;
               }
            }

            AnnotatedException currentDate1 = var10000;
            throw new AnnotatedException("Freshest CRL extension could not be decoded from certificate.", currentDate1);
         }

         CRLDistPoint var5 = var15;
         if (var15 == null) {
            try {
               var15 = CRLDistPoint.getInstance(CertPathValidatorUtilities.getExtensionValue(crl, cert1));
            } catch (AnnotatedException var8) {
               throw new AnnotatedException("Freshest CRL extension could not be decoded from CRL.", var8);
            }

            var5 = var15;
         }

         if (var5 != null) {
            HashSet var16;
            Date var17;
            ExtendedPKIXParameters var10002;
            X509CRL var10003;
            try {
               var16 = var4;
               var17 = currentDate;
               var10002 = paramsPKIX;
               var10003 = crl;
               CertPathValidatorUtilities.addAdditionalStoresFromCRLDistributionPoint(var5, paramsPKIX);
            } catch (AnnotatedException var7) {
               throw new AnnotatedException("No new delta CRL locations could be added from Freshest CRL extension.", var7);
            }

            try {
               var16.addAll(CertPathValidatorUtilities.getDeltaCRLs(var17, var10002, var10003));
            } catch (AnnotatedException var6) {
               throw new AnnotatedException("Exception obtaining delta CRLs.", var6);
            }
         }
      }

      return var4;
   }

   protected static Set[] processCRLA1ii(Date currentDate, ExtendedPKIXParameters paramsPKIX, X509Certificate cert, X509CRL crl) {
      HashSet var4;
      var4 = new HashSet.<init>();
      HashSet var5;
      var5 = new HashSet.<init>();
      X509CRLStoreSelector var6;
      X509CRLStoreSelector var10001 = var6 = new X509CRLStoreSelector;
      var6.<init>();
      var10001.setCertificateChecking(cert);
      if (paramsPKIX.getDate() != null) {
         var6.setDateAndTime(paramsPKIX.getDate());
      } else {
         var6.setDateAndTime(currentDate);
      }

      HashSet var14;
      ExtendedPKIXParameters var10000;
      X509CRLStoreSelector var10002;
      ExtendedPKIXParameters var10003;
      HashSet var10004;
      X509CRLStoreSelector var10005;
      ExtendedPKIXParameters var10006;
      HashSet var10007;
      X509CRLStoreSelector var10008;
      ExtendedPKIXParameters var10009;
      X509CRLStoreSelector var10010;
      try {
         var10000 = paramsPKIX;
         var14 = var4;
         var10002 = var6;
         var10003 = paramsPKIX;
         var10004 = var4;
         var10005 = var6;
         var10006 = paramsPKIX;
         var10007 = var4;
         var10008 = var6;
         var10009 = paramsPKIX;
         var10010 = var6;
         var6.addIssuerName(crl.getIssuerX500Principal().getEncoded());
      } catch (IOException var8) {
         throw new AnnotatedException("Cannot extract issuer from CRL." + var8, var8);
      }

      var10010.setCompleteCRLEnabled(true);

      label49: {
         AnnotatedException var13;
         label53: {
            boolean var15;
            try {
               var10007.addAll(CertPathValidatorUtilities.findCRLs(var10008, var10009.getAdditionalStores()));
            } catch (AnnotatedException var11) {
               var13 = var11;
               var15 = false;
               break label53;
            }

            try {
               var10004.addAll(CertPathValidatorUtilities.findCRLs(var10005, var10006.getStores()));
            } catch (AnnotatedException var10) {
               var13 = var10;
               var15 = false;
               break label53;
            }

            try {
               var14.addAll(CertPathValidatorUtilities.findCRLs(var10002, var10003.getCertStores()));
               break label49;
            } catch (AnnotatedException var9) {
               var13 = var9;
               var15 = false;
            }
         }

         AnnotatedException currentDate1 = var13;
         throw new AnnotatedException("Exception obtaining complete CRLs.", currentDate1);
      }

      if (var10000.isUseDeltasEnabled()) {
         try {
            var5.addAll(CertPathValidatorUtilities.getDeltaCRLs(currentDate, paramsPKIX, crl));
         } catch (AnnotatedException var7) {
            throw new AnnotatedException("Exception obtaining delta CRLs.", var7);
         }
      }

      return new Set[]{var4, var5};
   }

   protected static void processCRLC(X509CRL deltaCRL, X509CRL completeCRL, ExtendedPKIXParameters pkixParams) {
      if (deltaCRL != null) {
         String pkixParams1;
         ExtendedPKIXParameters var16;
         boolean var10001;
         IssuingDistributionPoint var20;
         label78: {
            Exception var10000;
            label83: {
               X509CRL var17;
               String var10002;
               try {
                  var16 = pkixParams;
                  var17 = completeCRL;
                  var10002 = ISSUING_DISTRIBUTION_POINT;
               } catch (Exception var9) {
                  var10000 = var9;
                  var10001 = false;
                  break label83;
               }

               pkixParams1 = var10002;

               try {
                  var20 = IssuingDistributionPoint.getInstance(CertPathValidatorUtilities.getExtensionValue(var17, var10002));
                  break label78;
               } catch (Exception var8) {
                  var10000 = var8;
                  var10001 = false;
               }
            }

            Exception deltaCRL1 = var10000;
            throw new AnnotatedException("Issuing distribution point extension could not be decoded.", deltaCRL1);
         }

         IssuingDistributionPoint var3 = var20;
         if (var16.isUseDeltasEnabled()) {
            if (!deltaCRL.getIssuerX500Principal().equals(completeCRL.getIssuerX500Principal())) {
               throw new AnnotatedException("Complete CRL issuer does not match delta CRL issuer.");
            }

            if (var3 != null) {
               IssuingDistributionPoint var18;
               try {
                  var18 = var3;
                  var20 = IssuingDistributionPoint.getInstance(CertPathValidatorUtilities.getExtensionValue(deltaCRL, pkixParams1));
               } catch (Exception var5) {
                  throw new AnnotatedException("Issuing distribution point extension from delta CRL could not be decoded.", var5);
               }

               if (!var18.equals(var20)) {
                  throw new AnnotatedException("Issuing distribution point extension from delta CRL and complete CRL does not match.");
               }
            }

            String completeCRL1;
            DERObject var23;
            label65: {
               AnnotatedException var19;
               label85: {
                  X509CRL var21;
                  String var22;
                  try {
                     var21 = deltaCRL;
                     var22 = AUTHORITY_KEY_IDENTIFIER;
                  } catch (AnnotatedException var7) {
                     var19 = var7;
                     var10001 = false;
                     break label85;
                  }

                  completeCRL1 = var22;

                  try {
                     var23 = CertPathValidatorUtilities.getExtensionValue(var21, var22);
                     break label65;
                  } catch (AnnotatedException var6) {
                     var19 = var6;
                     var10001 = false;
                  }
               }

               AnnotatedException deltaCRL2 = var19;
               throw new AnnotatedException("Authority key identifier extension could not be extracted from complete CRL.", deltaCRL2);
            }

            DERObject pkixParams2 = var23;

            DERObject var24;
            try {
               var24 = CertPathValidatorUtilities.getExtensionValue(deltaCRL, completeCRL1);
            } catch (AnnotatedException var4) {
               throw new AnnotatedException("Authority key identifier extension could not be extracted from delta CRL.", var4);
            }

            DERObject deltaCRL3 = var24;
            if ((var23 != null || deltaCRL3 != null) && !pkixParams2.equals(deltaCRL3)) {
               throw new AnnotatedException("Delta CRL authority key identifier does not match complete CRL authority key identifier.");
            }
         }

      }
   }

   protected static void processCRLI(Date validDate, X509CRL deltacrl, Object cert, CertStatus certStatus, ExtendedPKIXParameters pkixParams) {
      if (pkixParams.isUseDeltasEnabled() && deltacrl != null) {
         CertPathValidatorUtilities.getCertStatus(validDate, deltacrl, cert, certStatus);
      }

   }

   protected static void processCRLJ(Date validDate, X509CRL completecrl, Object cert, CertStatus certStatus) {
      if (certStatus.getCertStatus() == 11) {
         CertPathValidatorUtilities.getCertStatus(validDate, completecrl, cert, certStatus);
      }

   }

   protected static PKIXPolicyNode prepareCertB(CertPath certPath, int index, List[] policyNodes, PKIXPolicyNode validPolicyTree, int policyMapping) {
      List var5;
      X509Certificate var6;
      X509Certificate var10000 = var6 = (X509Certificate)(var5 = certPath.getCertificates()).get(index);
      int var20 = var5.size() - index;

      ASN1Sequence var36;
      try {
         var36 = ASN1Sequence.getInstance(CertPathValidatorUtilities.getExtensionValue(var10000, POLICY_MAPPINGS));
      } catch (AnnotatedException var19) {
         throw new ExtCertPathValidatorException("Policy mappings extension could not be decoded.", var19, certPath, index);
      }

      ASN1Sequence var7 = var36;
      if (var36 != null) {
         HashMap var8;
         var8 = new HashMap.<init>();
         HashSet var9;
         var9 = new HashSet.<init>();

         int var10;
         for(var10 = 0; var10 < var7.size(); ++var10) {
            ASN1Sequence var10001 = (ASN1Sequence)var7.getObjectAt(var10);
            String var11 = ((DERObjectIdentifier)var10001.getObjectAt(0)).getId();
            String var12 = ((DERObjectIdentifier)var10001.getObjectAt(1)).getId();
            if (!var8.containsKey(var11)) {
               HashSet var13;
               HashSet var10002 = var13 = new HashSet;
               var10002.<init>();
               var10002.add(var12);
               var8.put(var11, var13);
               var9.add(var11);
            } else {
               ((Set)var8.get(var11)).add(var12);
            }
         }

         Iterator var26 = var9.iterator();

         while(true) {
            while(true) {
               String var21;
               Iterator var22;
               boolean var23;
               PKIXPolicyNode var24;
               label119:
               do {
                  label108:
                  while(var26.hasNext()) {
                     var21 = (String)var26.next();
                     if (policyMapping > 0) {
                        var23 = false;
                        Iterator var27 = policyNodes[var20].iterator();

                        while(var27.hasNext()) {
                           PKIXPolicyNode var29;
                           if ((var29 = (PKIXPolicyNode)var27.next()).getValidPolicy().equals(var21)) {
                              var23 = true;
                              var29.expectedPolicies = (Set)var8.get(var21);
                              continue label119;
                           }
                        }
                        continue label119;
                     }

                     if (policyMapping <= 0) {
                        var22 = policyNodes[var20].iterator();

                        while(true) {
                           do {
                              if (!var22.hasNext()) {
                                 continue label108;
                              }
                           } while(!(var24 = (PKIXPolicyNode)var22.next()).getValidPolicy().equals(var21));

                           ((PKIXPolicyNode)var24.getParent()).removeChild(var24);
                           var22.remove();

                           for(var10 = var20 - 1; var10 >= 0; --var10) {
                              List var28 = policyNodes[var10];

                              PKIXPolicyNode var14;
                              for(int var31 = 0; var31 < var28.size() && ((var14 = (PKIXPolicyNode)var28.get(var31)).hasChildren() || (validPolicyTree = CertPathValidatorUtilities.removePolicyNode(validPolicyTree, policyNodes, var14)) != null); ++var31) {
                              }
                           }
                        }
                     }
                  }

                  return validPolicyTree;
               } while(var23);

               var22 = policyNodes[var20].iterator();

               while(var22.hasNext()) {
                  if ("2.5.29.32.0".equals((var24 = (PKIXPolicyNode)var22.next()).getValidPolicy())) {
                     var10000 = var6;
                     Set var25 = null;

                     try {
                        var36 = (ASN1Sequence)CertPathValidatorUtilities.getExtensionValue(var10000, CERTIFICATE_POLICIES);
                     } catch (AnnotatedException var18) {
                        throw new ExtCertPathValidatorException("Certificate policies extension could not be decoded.", var18, certPath, index);
                     }

                     Enumeration var30 = var36.getObjects();

                     while(var30.hasMoreElements()) {
                        PolicyInformation var37;
                        try {
                           var37 = PolicyInformation.getInstance(var30.nextElement());
                        } catch (Exception var17) {
                           throw new CertPathValidatorException("Policy information could not be decoded.", var17, certPath, index);
                        }

                        PolicyInformation var33 = var37;
                        if ("2.5.29.32.0".equals(var37.getPolicyIdentifier().getId())) {
                           Set var38;
                           try {
                              var38 = CertPathValidatorUtilities.getQualifierSet(var33.getPolicyQualifiers());
                           } catch (CertPathValidatorException var16) {
                              throw new ExtCertPathValidatorException("Policy qualifier info set could not be decoded.", var16, certPath, index);
                           }

                           var25 = var38;
                           break;
                        }
                     }

                     boolean var32 = false;
                     if (var6.getCriticalExtensionOIDs() != null) {
                        var32 = var6.getCriticalExtensionOIDs().contains(CERTIFICATE_POLICIES);
                     }

                     if ("2.5.29.32.0".equals((var24 = (PKIXPolicyNode)var24.getParent()).getValidPolicy())) {
                        PKIXPolicyNode var34;
                        PKIXPolicyNode var10003 = var34 = new PKIXPolicyNode;
                        ArrayList var35;
                        var35 = new ArrayList.<init>();
                        Set var15 = (Set)var8.get(var21);
                        var10003.<init>(var35, var20, var15, var24, var25, var21, var32);
                        var24.addChild(var10003);
                        policyNodes[var20].add(var34);
                     }
                     break;
                  }
               }
            }
         }
      } else {
         return validPolicyTree;
      }
   }

   protected static void prepareNextCertA(CertPath certPath, int index) {
      X509Certificate var10000 = (X509Certificate)certPath.getCertificates().get(index);

      ASN1Sequence var10;
      try {
         var10 = ASN1Sequence.getInstance(CertPathValidatorUtilities.getExtensionValue(var10000, POLICY_MAPPINGS));
      } catch (AnnotatedException var5) {
         throw new ExtCertPathValidatorException("Policy mappings extension could not be decoded.", var5, certPath, index);
      }

      ASN1Sequence var2 = var10;
      if (var10 != null) {
         for(int var3 = 0; var3 < var2.size(); ++var3) {
            DERObjectIdentifier var12;
            DERObjectIdentifier var13;
            label45: {
               Exception var11;
               label55: {
                  boolean var10001;
                  try {
                     var10 = ASN1Sequence.getInstance(var2.getObjectAt(var3));
                  } catch (Exception var7) {
                     var11 = var7;
                     var10001 = false;
                     break label55;
                  }

                  ASN1Sequence var4 = var10;

                  try {
                     var13 = DERObjectIdentifier.getInstance(var10.getObjectAt(0));
                     var12 = DERObjectIdentifier.getInstance(var4.getObjectAt(1));
                     break label45;
                  } catch (Exception var6) {
                     var11 = var6;
                     var10001 = false;
                  }
               }

               Exception var8 = var11;
               throw new ExtCertPathValidatorException("Policy mappings extension contents could not be decoded.", var8, certPath, index);
            }

            DERObjectIdentifier var9 = var12;
            if ("2.5.29.32.0".equals(var13.getId())) {
               throw new CertPathValidatorException("IssuerDomainPolicy is anyPolicy", (Throwable)null, certPath, index);
            }

            if ("2.5.29.32.0".equals(var9.getId())) {
               throw new CertPathValidatorException("SubjectDomainPolicy is anyPolicy,", (Throwable)null, certPath, index);
            }
         }
      }

   }

   protected static void processCertF(CertPath certPath, int index, PKIXPolicyNode validPolicyTree, int explicitPolicy) {
      if (explicitPolicy <= 0 && validPolicyTree == null) {
         throw new ExtCertPathValidatorException("No valid policy tree found when one expected.", (Throwable)null, certPath, index);
      }
   }

   protected static PKIXPolicyNode processCertE(CertPath certPath, int index, PKIXPolicyNode validPolicyTree) {
      X509Certificate var10000 = (X509Certificate)certPath.getCertificates().get(index);

      ASN1Sequence var4;
      try {
         var4 = ASN1Sequence.getInstance(CertPathValidatorUtilities.getExtensionValue(var10000, CERTIFICATE_POLICIES));
      } catch (AnnotatedException var3) {
         throw new ExtCertPathValidatorException("Could not read certificate policies extension from certificate.", var3, certPath, index);
      }

      if (var4 == null) {
         validPolicyTree = null;
      }

      return validPolicyTree;
   }

   protected static void processCertBC(CertPath certPath, int index, PKIXNameConstraintValidator nameConstraintValidator) {
      List var10000 = certPath.getCertificates();
      X509Certificate var3 = (X509Certificate)var10000.get(index);
      int var4 = var10000.size();
      int var5 = var4 - index;
      if (!CertPathValidatorUtilities.isSelfIssued(var3) || var5 >= var4) {
         X509Certificate var19 = var3;
         PKIXNameConstraintValidator var10001 = nameConstraintValidator;
         PKIXNameConstraintValidator var10002 = nameConstraintValidator;
         X500Principal var13 = CertPathValidatorUtilities.getSubjectPrincipal(var3);
         ASN1InputStream var10003 = new ASN1InputStream(var13.getEncoded());

         ASN1Sequence var24;
         try {
            var24 = ASN1Sequence.getInstance(var10003.readObject());
         } catch (Exception var12) {
            throw new CertPathValidatorException("Exception extracting subject name when checking subtrees.", var12, certPath, index);
         }

         ASN1Sequence var14 = var24;

         try {
            var10002.checkPermittedDN(var24);
            var10001.checkExcludedDN(var14);
         } catch (PKIXNameConstraintValidatorException var11) {
            throw new CertPathValidatorException("Subtree check for certificate subject failed.", var11, certPath, index);
         }

         GeneralNames var20;
         try {
            var20 = GeneralNames.getInstance(CertPathValidatorUtilities.getExtensionValue(var19, SUBJECT_ALTERNATIVE_NAME));
         } catch (Exception var10) {
            throw new CertPathValidatorException("Subject alternative name extension could not be decoded.", var10, certPath, index);
         }

         GeneralNames var16 = var20;
         Enumeration var15 = (new X509Name(var14)).getValues(X509Name.EmailAddress).elements();

         while(var15.hasMoreElements()) {
            PKIXNameConstraintValidator var21 = nameConstraintValidator;
            var10001 = nameConstraintValidator;
            String var18 = (String)var15.nextElement();
            GeneralName var6;
            GeneralName var23 = var6 = new GeneralName;
            var23.<init>(1, var18);

            try {
               var10001.checkPermitted(var23);
               var21.checkExcluded(var6);
            } catch (PKIXNameConstraintValidatorException var9) {
               throw new CertPathValidatorException("Subtree check for certificate subject alternative email failed.", var9, certPath, index);
            }
         }

         if (var16 != null) {
            GeneralName[] var22;
            try {
               var22 = var16.getNames();
            } catch (Exception var8) {
               throw new CertPathValidatorException("Subject alternative name contents could not be decoded.", var8, certPath, index);
            }

            GeneralName[] var17 = var22;

            for(var4 = 0; var4 < var17.length; ++var4) {
               try {
                  nameConstraintValidator.checkPermitted(var17[var4]);
                  nameConstraintValidator.checkExcluded(var17[var4]);
               } catch (PKIXNameConstraintValidatorException var7) {
                  throw new CertPathValidatorException("Subtree check for certificate subject alternative name failed.", var7, certPath, index);
               }
            }
         }
      }

   }

   protected static PKIXPolicyNode processCertD(CertPath certPath, int index, Set acceptablePolicies, PKIXPolicyNode validPolicyTree, List[] policyNodes, int inhibitAnyPolicy) {
      List var6;
      X509Certificate var7;
      X509Certificate var10000 = var7 = (X509Certificate)(var6 = certPath.getCertificates()).get(index);
      int var26;
      int var8 = (var26 = var6.size()) - index;

      ASN1Sequence var37;
      try {
         var37 = ASN1Sequence.getInstance(CertPathValidatorUtilities.getExtensionValue(var10000, CERTIFICATE_POLICIES));
      } catch (AnnotatedException var15) {
         throw new ExtCertPathValidatorException("Could not read certificate policies extension from certificate.", var15, certPath, index);
      }

      ASN1Sequence var9 = var37;
      if (var37 != null && validPolicyTree != null) {
         Enumeration var10 = var9.getObjects();
         HashSet var11;
         var11 = new HashSet.<init>();

         while(var10.hasMoreElements()) {
            PolicyInformation var12;
            DERObjectIdentifier var13;
            DERObjectIdentifier var38 = var13 = (var12 = PolicyInformation.getInstance(var10.nextElement())).getPolicyIdentifier();
            var11.add(var13.getId());
            if (!"2.5.29.32.0".equals(var38.getId())) {
               List[] var10001;
               DERObjectIdentifier var10002;
               Set var10003;
               int var39;
               try {
                  var39 = var8;
                  var10001 = policyNodes;
                  var10002 = var13;
                  var10003 = CertPathValidatorUtilities.getQualifierSet(var12.getPolicyQualifiers());
               } catch (CertPathValidatorException var14) {
                  throw new ExtCertPathValidatorException("Policy qualifier info set could not be build.", var14, certPath, index);
               }

               Set var35 = var10003;
               if (!CertPathValidatorUtilities.processCertD1i(var39, var10001, var10002, var10003)) {
                  CertPathValidatorUtilities.processCertD1ii(var8, policyNodes, var13, var35);
               }
            }
         }

         if (!acceptablePolicies.isEmpty() && !acceptablePolicies.contains("2.5.29.32.0")) {
            Iterator certPath1 = acceptablePolicies.iterator();
            HashSet index1;
            index1 = new HashSet.<init>();

            while(certPath1.hasNext()) {
               Object var30;
               if (var11.contains(var30 = certPath1.next())) {
                  index1.add(var30);
               }
            }

            acceptablePolicies.clear();
            acceptablePolicies.addAll(index1);
         } else {
            acceptablePolicies.clear();
            acceptablePolicies.addAll(var11);
         }

         Set certPath3;
         List index2;
         int acceptablePolicies1;
         if (inhibitAnyPolicy > 0 || var8 < var26 && CertPathValidatorUtilities.isSelfIssued(var7)) {
            Enumeration var36 = var9.getObjects();

            label123:
            while(var36.hasMoreElements()) {
               PolicyInformation certPath2;
               if ("2.5.29.32.0".equals((certPath2 = PolicyInformation.getInstance(var36.nextElement())).getPolicyIdentifier().getId())) {
                  certPath3 = CertPathValidatorUtilities.getQualifierSet(certPath2.getPolicyQualifiers());
                  index2 = policyNodes[var8 - 1];
                  acceptablePolicies1 = 0;

                  label117:
                  while(true) {
                     if (acceptablePolicies1 >= index2.size()) {
                        break label123;
                     }

                     PKIXPolicyNode inhibitAnyPolicy1;
                     Iterator var27 = (inhibitAnyPolicy1 = (PKIXPolicyNode)index2.get(acceptablePolicies1)).getExpectedPolicies().iterator();

                     while(true) {
                        String var29;
                        while(true) {
                           if (!var27.hasNext()) {
                              ++acceptablePolicies1;
                              continue label117;
                           }

                           Object var28;
                           if ((var28 = var27.next()) instanceof String) {
                              var29 = (String)var28;
                              break;
                           }

                           if (var28 instanceof DERObjectIdentifier) {
                              var29 = ((DERObjectIdentifier)var28).getId();
                              break;
                           }
                        }

                        boolean var31 = false;
                        Iterator var32 = inhibitAnyPolicy1.getChildren();

                        while(var32.hasNext()) {
                           if (var29.equals(((PKIXPolicyNode)var32.next()).getValidPolicy())) {
                              var31 = true;
                           }
                        }

                        if (!var31) {
                           HashSet var33;
                           HashSet var40 = var33 = new HashSet;
                           var33.<init>();
                           var40.add(var29);
                           PKIXPolicyNode var34;
                           PKIXPolicyNode var41 = var34 = new PKIXPolicyNode;
                           var41.<init>(new ArrayList(), var8, var33, inhibitAnyPolicy1, certPath3, var29, false);
                           inhibitAnyPolicy1.addChild(var41);
                           policyNodes[var8].add(var34);
                        }
                     }
                  }
               }
            }
         }

         int certPath4 = var8 - 1;

         for(inhibitAnyPolicy = certPath4; inhibitAnyPolicy >= 0; inhibitAnyPolicy = certPath4) {
            var6 = policyNodes[inhibitAnyPolicy];

            PKIXPolicyNode index3;
            for(certPath4 = 0; certPath4 < var6.size() && ((index3 = (PKIXPolicyNode)var6.get(certPath4)).hasChildren() || (validPolicyTree = CertPathValidatorUtilities.removePolicyNode(validPolicyTree, policyNodes, index3)) != null); ++certPath4) {
            }

            certPath4 = inhibitAnyPolicy + -1;
         }

         if ((certPath3 = var7.getCriticalExtensionOIDs()) != null) {
            boolean certPath5 = certPath3.contains(CERTIFICATE_POLICIES);
            index2 = policyNodes[var8];

            for(acceptablePolicies1 = 0; acceptablePolicies1 < index2.size(); ++acceptablePolicies1) {
               ((PKIXPolicyNode)index2.get(acceptablePolicies1)).setCritical(certPath5);
            }
         }

         return validPolicyTree;
      } else {
         return null;
      }
   }

   protected static void processCertA(CertPath certPath, ExtendedPKIXParameters paramsPKIX, int index, PublicKey workingPublicKey, X500Principal workingIssuerName, X509Certificate sign) {
      List var6;
      X509Certificate var7;
      X509Certificate var10001 = var7 = (X509Certificate)(var6 = certPath.getCertificates()).get(index);

      ExtendedPKIXParameters var10002;
      CertPath var10003;
      int var10004;
      try {
         var10002 = paramsPKIX;
         var10003 = certPath;
         var10004 = index;
         var7.verify(workingPublicKey, "INFOSEC");
      } catch (GeneralSecurityException var12) {
         throw new ExtCertPathValidatorException("Could not validate certificate signature.", var12, certPath, index);
      }

      try {
         var10001.checkValidity(CertPathValidatorUtilities.getValidCertDateFromValidityModel(var10002, var10003, var10004));
      } catch (CertificateExpiredException var9) {
         throw new ExtCertPathValidatorException("Could not validate certificate: " + var9.getMessage(), var9, certPath, index);
      } catch (CertificateNotYetValidException var10) {
         throw new ExtCertPathValidatorException("Could not validate certificate: " + var10.getMessage(), var10, certPath, index);
      } catch (AnnotatedException var11) {
         throw new ExtCertPathValidatorException("Could not validate time of certificate.", var11, certPath, index);
      }

      if (paramsPKIX.isRevocationEnabled()) {
         try {
            checkCRLs(paramsPKIX, var7, CertPathValidatorUtilities.getValidCertDateFromValidityModel(paramsPKIX, certPath, index), sign, workingPublicKey, var6);
         } catch (AnnotatedException var8) {
            String paramsPKIX1 = var8.getMessage();
            throw new ExtCertPathValidatorException(paramsPKIX1, var8.getCause(), certPath, index);
         }
      }

      if (!CertPathValidatorUtilities.getEncodedIssuerPrincipal(var7).equals(workingIssuerName)) {
         throw new ExtCertPathValidatorException("IssuerName(" + CertPathValidatorUtilities.getEncodedIssuerPrincipal(var7) + ") does not match SubjectName(" + workingIssuerName + ") of signing certificate.", (Throwable)null, certPath, index);
      }
   }

   protected static int prepareNextCertI1(CertPath certPath, int index, int explicitPolicy) {
      X509Certificate var10000 = (X509Certificate)certPath.getCertificates().get(index);

      ASN1Sequence var12;
      try {
         var12 = ASN1Sequence.getInstance(CertPathValidatorUtilities.getExtensionValue(var10000, POLICY_CONSTRAINTS));
      } catch (Exception var5) {
         throw new ExtCertPathValidatorException("Policy constraints extension cannot be decoded.", var5, certPath, index);
      }

      ASN1Sequence var3 = var12;
      if (var12 != null) {
         Enumeration var11 = var3.getObjects();

         int var15;
         while(true) {
            if (!var11.hasMoreElements()) {
               return explicitPolicy;
            }

            IllegalArgumentException var13;
            label58: {
               ASN1TaggedObject var14;
               boolean var10001;
               try {
                  var14 = ASN1TaggedObject.getInstance(var11.nextElement());
               } catch (IllegalArgumentException var8) {
                  var13 = var8;
                  var10001 = false;
                  break label58;
               }

               ASN1TaggedObject var4 = var14;

               try {
                  var15 = var14.getTagNo();
               } catch (IllegalArgumentException var7) {
                  var13 = var7;
                  var10001 = false;
                  break label58;
               }

               if (var15 != 0) {
                  continue;
               }

               try {
                  var15 = DERInteger.getInstance(var4).getValue().intValue();
                  break;
               } catch (IllegalArgumentException var6) {
                  var13 = var6;
                  var10001 = false;
               }
            }

            IllegalArgumentException explicitPolicy1 = var13;
            throw new ExtCertPathValidatorException("Policy constraints extension contents cannot be decoded.", explicitPolicy1, certPath, index);
         }

         int certPath1 = var15;
         if (var15 < explicitPolicy) {
            return certPath1;
         }
      }

      return explicitPolicy;
   }

   protected static int prepareNextCertI2(CertPath certPath, int index, int policyMapping) {
      X509Certificate var10000 = (X509Certificate)certPath.getCertificates().get(index);

      ASN1Sequence var12;
      try {
         var12 = ASN1Sequence.getInstance(CertPathValidatorUtilities.getExtensionValue(var10000, POLICY_CONSTRAINTS));
      } catch (Exception var5) {
         throw new ExtCertPathValidatorException("Policy constraints extension cannot be decoded.", var5, certPath, index);
      }

      ASN1Sequence var3 = var12;
      if (var12 != null) {
         Enumeration var11 = var3.getObjects();

         int var15;
         while(true) {
            if (!var11.hasMoreElements()) {
               return policyMapping;
            }

            IllegalArgumentException var13;
            label58: {
               ASN1TaggedObject var14;
               boolean var10001;
               try {
                  var14 = ASN1TaggedObject.getInstance(var11.nextElement());
               } catch (IllegalArgumentException var8) {
                  var13 = var8;
                  var10001 = false;
                  break label58;
               }

               ASN1TaggedObject var4 = var14;

               try {
                  var15 = var14.getTagNo();
               } catch (IllegalArgumentException var7) {
                  var13 = var7;
                  var10001 = false;
                  break label58;
               }

               if (var15 != 1) {
                  continue;
               }

               try {
                  var15 = DERInteger.getInstance(var4).getValue().intValue();
                  break;
               } catch (IllegalArgumentException var6) {
                  var13 = var6;
                  var10001 = false;
               }
            }

            IllegalArgumentException policyMapping1 = var13;
            throw new ExtCertPathValidatorException("Policy constraints extension contents cannot be decoded.", policyMapping1, certPath, index);
         }

         int certPath1 = var15;
         if (var15 < policyMapping) {
            return certPath1;
         }
      }

      return policyMapping;
   }

   protected static void prepareNextCertG(CertPath certPath, int index, PKIXNameConstraintValidator nameConstraintValidator) {
      X509Certificate var10000 = (X509Certificate)certPath.getCertificates().get(index);
      NameConstraints var3 = null;

      ASN1Sequence var4;
      Exception nameConstraintValidator1;
      Exception var14;
      boolean var10001;
      label67: {
         label71: {
            ASN1Sequence var15;
            try {
               var15 = ASN1Sequence.getInstance(CertPathValidatorUtilities.getExtensionValue(var10000, NAME_CONSTRAINTS));
            } catch (Exception var10) {
               var14 = var10;
               var10001 = false;
               break label71;
            }

            var4 = var15;
            if (var15 == null) {
               break label67;
            }

            NameConstraints var16;
            try {
               var16 = new NameConstraints;
            } catch (Exception var9) {
               var14 = var9;
               var10001 = false;
               break label71;
            }

            var3 = var16;

            try {
               var16.<init>(var4);
               break label67;
            } catch (Exception var8) {
               var14 = var8;
               var10001 = false;
            }
         }

         nameConstraintValidator1 = var14;
         throw new ExtCertPathValidatorException("Name constraints extension could not be decoded.", nameConstraintValidator1, certPath, index);
      }

      if (var3 != null) {
         if ((var4 = var3.getPermittedSubtrees()) != null) {
            try {
               nameConstraintValidator.intersectPermittedSubtree(var4);
            } catch (Exception var5) {
               throw new ExtCertPathValidatorException("Permitted subtrees cannot be build from name constraints extension.", var5, certPath, index);
            }
         }

         ASN1Sequence var12;
         if ((var12 = var3.getExcludedSubtrees()) != null) {
            Enumeration var13 = var12.getObjects();

            while(true) {
               boolean var17;
               try {
                  var17 = var13.hasMoreElements();
               } catch (Exception var7) {
                  var14 = var7;
                  var10001 = false;
                  break;
               }

               if (!var17) {
                  return;
               }

               try {
                  nameConstraintValidator.addExcludedSubtree(GeneralSubtree.getInstance(var13.nextElement()));
               } catch (Exception var6) {
                  var14 = var6;
                  var10001 = false;
                  break;
               }
            }

            nameConstraintValidator1 = var14;
            throw new ExtCertPathValidatorException("Excluded subtrees cannot be build from name constraints extension.", nameConstraintValidator1, certPath, index);
         }
      }

   }

   private static void checkCRL(DistributionPoint dp, ExtendedPKIXParameters paramsPKIX, X509Certificate cert, Date validDate, X509Certificate defaultCRLSignCert, PublicKey defaultCRLSignKey, CertStatus certStatus, ReasonsMask reasonMask, List certPathCerts) {
      Date var9;
      var9 = new Date.<init>(System.currentTimeMillis());
      if (validDate.getTime() > var9.getTime()) {
         throw new AnnotatedException("Validation time is in future.");
      } else {
         Set var10000 = CertPathValidatorUtilities.getCompleteCRLs(dp, cert, var9, paramsPKIX);
         boolean var10 = false;
         AnnotatedException var11 = null;
         Iterator var12 = var10000.iterator();

         while(var12.hasNext() && certStatus.getCertStatus() == 11 && !reasonMask.isAllReasons()) {
            AnnotatedException var39;
            label180: {
               boolean var10001;
               X509CRL var40;
               try {
                  var40 = (X509CRL)var12.next();
               } catch (AnnotatedException var37) {
                  var39 = var37;
                  var10001 = false;
                  break label180;
               }

               X509CRL var13 = var40;

               ReasonsMask var41;
               try {
                  var41 = processCRLD(var40, dp);
               } catch (AnnotatedException var36) {
                  var39 = var36;
                  var10001 = false;
                  break label180;
               }

               ReasonsMask var14 = var41;

               boolean var42;
               try {
                  var42 = var41.hasNewReasons(reasonMask);
               } catch (AnnotatedException var35) {
                  var39 = var35;
                  var10001 = false;
                  break label180;
               }

               if (!var42) {
                  continue;
               }

               PublicKey var43;
               ExtendedPKIXParameters var44;
               try {
                  var44 = paramsPKIX;
                  var43 = processCRLG(var13, processCRLF(var13, cert, defaultCRLSignCert, defaultCRLSignKey, paramsPKIX, certPathCerts));
               } catch (AnnotatedException var34) {
                  var39 = var34;
                  var10001 = false;
                  break label180;
               }

               PublicKey var15 = var43;
               X509CRL var16 = null;

               try {
                  var42 = var44.isUseDeltasEnabled();
               } catch (AnnotatedException var33) {
                  var39 = var33;
                  var10001 = false;
                  break label180;
               }

               if (var42) {
                  try {
                     var40 = processCRLH(CertPathValidatorUtilities.getDeltaCRLs(var9, paramsPKIX, var13), var15);
                  } catch (AnnotatedException var32) {
                     var39 = var32;
                     var10001 = false;
                     break label180;
                  }

                  var16 = var40;
               }

               int var46;
               try {
                  var46 = paramsPKIX.getValidityModel();
               } catch (AnnotatedException var31) {
                  var39 = var31;
                  var10001 = false;
                  break label180;
               }

               if (var46 != 1) {
                  long var45;
                  long var47;
                  try {
                     var47 = cert.getNotAfter().getTime();
                     var45 = var13.getThisUpdate().getTime();
                  } catch (AnnotatedException var30) {
                     var39 = var30;
                     var10001 = false;
                     break label180;
                  }

                  if (var47 < var45) {
                     try {
                        throw new AnnotatedException("No valid CRL for current time found.");
                     } catch (AnnotatedException var17) {
                        var39 = var17;
                        var10001 = false;
                        break label180;
                     }
                  }
               }

               try {
                  processCRLB1(dp, cert, var13);
                  processCRLB2(dp, cert, var13);
                  processCRLC(var16, var13, paramsPKIX);
                  processCRLI(validDate, var16, cert, certStatus, paramsPKIX);
                  processCRLJ(validDate, var13, cert, certStatus);
                  var46 = certStatus.getCertStatus();
               } catch (AnnotatedException var29) {
                  var39 = var29;
                  var10001 = false;
                  break label180;
               }

               if (var46 == 8) {
                  try {
                     certStatus.setCertStatus(11);
                  } catch (AnnotatedException var28) {
                     var39 = var28;
                     var10001 = false;
                     break label180;
                  }
               }

               try {
                  reasonMask.addReasons(var14);
                  var10000 = var13.getCriticalExtensionOIDs();
               } catch (AnnotatedException var27) {
                  var39 = var27;
                  var10001 = false;
                  break label180;
               }

               Set var38 = var10000;
               HashSet var48;
               HashSet var49;
               if (var10000 != null) {
                  try {
                     var49 = new HashSet;
                     var48 = var49;
                     var49.<init>(var38);
                     var49.remove(X509Extensions.IssuingDistributionPoint.getId());
                  } catch (AnnotatedException var26) {
                     var39 = var26;
                     var10001 = false;
                     break label180;
                  }

                  try {
                     var48.remove(X509Extensions.DeltaCRLIndicator.getId());
                  } catch (AnnotatedException var25) {
                     var39 = var25;
                     var10001 = false;
                     break label180;
                  }

                  try {
                     var42 = var49.isEmpty();
                  } catch (AnnotatedException var24) {
                     var39 = var24;
                     var10001 = false;
                     break label180;
                  }

                  if (!var42) {
                     try {
                        throw new AnnotatedException("CRL contains unsupported critical extensions.");
                     } catch (AnnotatedException var18) {
                        var39 = var18;
                        var10001 = false;
                        break label180;
                     }
                  }
               }

               if (var16 != null) {
                  try {
                     var10000 = var16.getCriticalExtensionOIDs();
                  } catch (AnnotatedException var23) {
                     var39 = var23;
                     var10001 = false;
                     break label180;
                  }

                  var38 = var10000;
                  if (var10000 != null) {
                     try {
                        var49 = new HashSet;
                        var48 = var49;
                        var49.<init>(var38);
                        var49.remove(X509Extensions.IssuingDistributionPoint.getId());
                     } catch (AnnotatedException var22) {
                        var39 = var22;
                        var10001 = false;
                        break label180;
                     }

                     try {
                        var48.remove(X509Extensions.DeltaCRLIndicator.getId());
                     } catch (AnnotatedException var21) {
                        var39 = var21;
                        var10001 = false;
                        break label180;
                     }

                     try {
                        var42 = var49.isEmpty();
                     } catch (AnnotatedException var20) {
                        var39 = var20;
                        var10001 = false;
                        break label180;
                     }

                     if (!var42) {
                        try {
                           throw new AnnotatedException("Delta CRL contains unsupported critical extension.");
                        } catch (AnnotatedException var19) {
                           var39 = var19;
                           var10001 = false;
                           break label180;
                        }
                     }
                  }
               }

               var10 = true;
               continue;
            }

            var11 = var39;
         }

         if (!var10) {
            throw var11;
         }
      }
   }

   protected static void checkCRLs(ExtendedPKIXParameters paramsPKIX, X509Certificate cert, Date validDate, X509Certificate sign, PublicKey workingPublicKey, List certPathCerts) {
      X509Certificate var10000 = cert;
      AnnotatedException var6 = null;

      CRLDistPoint var27;
      try {
         var27 = CRLDistPoint.getInstance(CertPathValidatorUtilities.getExtensionValue(var10000, CRL_DISTRIBUTION_POINTS));
      } catch (Exception var15) {
         throw new AnnotatedException("CRL distribution point extension could not be read.", var15);
      }

      CRLDistPoint var7;
      CRLDistPoint var10001 = var7 = var27;

      try {
         CertPathValidatorUtilities.addAdditionalStoresFromCRLDistributionPoint(var10001, paramsPKIX);
      } catch (AnnotatedException var14) {
         throw new AnnotatedException("No additional CRL locations could be decoded from CRL distribution point extension.", var14);
      }

      CertStatus var8;
      var8 = new CertStatus.<init>();
      ReasonsMask var9;
      var9 = new ReasonsMask.<init>();
      boolean var10 = false;
      if (var27 != null) {
         DistributionPoint[] var28;
         try {
            var28 = var7.getDistributionPoints();
         } catch (Exception var13) {
            throw new AnnotatedException("Distribution points could not be read.", var13);
         }

         DistributionPoint[] var25 = var28;
         if (var28 != null) {
            for(int var11 = 0; var11 < var25.length && var8.getCertStatus() == 11 && !var9.isAllReasons(); ++var11) {
               var28 = var25;
               int var30 = var11;
               ExtendedPKIXParameters var12 = (ExtendedPKIXParameters)paramsPKIX.clone();

               try {
                  checkCRL(var28[var30], var12, cert, validDate, sign, workingPublicKey, var8, var9, certPathCerts);
               } catch (AnnotatedException var21) {
                  var6 = var21;
                  continue;
               }

               var10 = true;
            }
         }
      }

      AnnotatedException paramsPKIX1;
      if (var8.getCertStatus() == 11 && !var9.isAllReasons()) {
         label101: {
            label100: {
               AnnotatedException var29;
               label128: {
                  DERObject var31;
                  boolean var32;
                  try {
                     var31 = (new ASN1InputStream(CertPathValidatorUtilities.getEncodedIssuerPrincipal(cert).getEncoded())).readObject();
                  } catch (Exception var20) {
                     Exception paramsPKIX2 = var20;

                     try {
                        throw new AnnotatedException("Issuer from certificate for CRL could not be reencoded.", paramsPKIX2);
                     } catch (AnnotatedException var16) {
                        var29 = var16;
                        var32 = false;
                        break label128;
                     }
                  }

                  DERObject var26 = var31;

                  DistributionPoint var33;
                  try {
                     var33 = new DistributionPoint;
                  } catch (AnnotatedException var19) {
                     var29 = var19;
                     var32 = false;
                     break label128;
                  }

                  DistributionPoint var34 = var33;

                  DistributionPoint var10002;
                  DistributionPointName var10003;
                  ExtendedPKIXParameters var35;
                  try {
                     var10002 = var34;
                     var35 = paramsPKIX;
                     var10003 = new DistributionPointName;
                  } catch (AnnotatedException var18) {
                     var29 = var18;
                     var32 = false;
                     break label128;
                  }

                  DistributionPointName var10004 = var10003;
                  byte var10005 = 0;

                  try {
                     var10004.<init>(var10005, new GeneralNames(new GeneralName(4, var26)));
                     var10002.<init>(var10003, (ReasonFlags)null, (GeneralNames)null);
                     checkCRL(var33, (ExtendedPKIXParameters)var35.clone(), cert, validDate, sign, workingPublicKey, var8, var9, certPathCerts);
                     break label100;
                  } catch (AnnotatedException var17) {
                     var29 = var17;
                     var32 = false;
                  }
               }

               paramsPKIX1 = var29;
               break label101;
            }

            var10 = true;
            paramsPKIX1 = var6;
         }
      } else {
         paramsPKIX1 = var6;
      }

      if (!var10) {
         if (paramsPKIX1 instanceof AnnotatedException) {
            throw paramsPKIX1;
         } else {
            throw new AnnotatedException("No valid CRL found.", paramsPKIX1);
         }
      } else if (var8.getCertStatus() == 11) {
         if (!var9.isAllReasons() && var8.getCertStatus() == 11) {
            var8.setCertStatus(12);
         }

         if (var8.getCertStatus() == 12) {
            throw new AnnotatedException("Certificate status could not be determined.");
         }
      } else {
         String paramsPKIX3 = "Certificate revocation after " + var8.getRevocationDate();
         paramsPKIX3 = paramsPKIX3 + ", reason: " + crlReasons[var8.getCertStatus()];
         throw new AnnotatedException(paramsPKIX3);
      }
   }

   protected static int prepareNextCertJ(CertPath certPath, int index, int inhibitAnyPolicy) {
      X509Certificate var10000 = (X509Certificate)certPath.getCertificates().get(index);

      DERInteger var4;
      try {
         var4 = DERInteger.getInstance(CertPathValidatorUtilities.getExtensionValue(var10000, INHIBIT_ANY_POLICY));
      } catch (Exception var3) {
         throw new ExtCertPathValidatorException("Inhibit any-policy extension cannot be decoded.", var3, certPath, index);
      }

      DERInteger certPath1 = var4;
      int certPath2;
      return var4 != null && (certPath2 = certPath1.getValue().intValue()) < inhibitAnyPolicy ? certPath2 : inhibitAnyPolicy;
   }

   protected static void prepareNextCertK(CertPath certPath, int index) {
      X509Certificate var10000 = (X509Certificate)certPath.getCertificates().get(index);

      BasicConstraints var4;
      try {
         var4 = BasicConstraints.getInstance(CertPathValidatorUtilities.getExtensionValue(var10000, BASIC_CONSTRAINTS));
      } catch (Exception var3) {
         throw new ExtCertPathValidatorException("Basic constraints extension cannot be decoded.", var3, certPath, index);
      }

      BasicConstraints certPath1 = var4;
      if (var4 != null) {
         if (!certPath1.isCA()) {
            throw new CertPathValidatorException("Not a CA certificate");
         }
      } else {
         throw new CertPathValidatorException("Intermediate certificate lacks BasicConstraints");
      }
   }

   protected static int prepareNextCertL(CertPath certPath, int index, int maxPathLength) {
      if (!CertPathValidatorUtilities.isSelfIssued((X509Certificate)certPath.getCertificates().get(index))) {
         if (maxPathLength > 0) {
            return maxPathLength - 1;
         } else {
            throw new ExtCertPathValidatorException("Max path length not greater than zero", (Throwable)null, certPath, index);
         }
      } else {
         return maxPathLength;
      }
   }

   protected static int prepareNextCertM(CertPath certPath, int index, int maxPathLength) {
      X509Certificate var10000 = (X509Certificate)certPath.getCertificates().get(index);

      BasicConstraints var4;
      try {
         var4 = BasicConstraints.getInstance(CertPathValidatorUtilities.getExtensionValue(var10000, BASIC_CONSTRAINTS));
      } catch (Exception var3) {
         throw new ExtCertPathValidatorException("Basic constraints extension cannot be decoded.", var3, certPath, index);
      }

      BasicConstraints certPath1 = var4;
      BigInteger certPath2;
      int certPath3;
      return var4 != null && (certPath2 = certPath1.getPathLenConstraint()) != null && (certPath3 = certPath2.intValue()) < maxPathLength ? certPath3 : maxPathLength;
   }

   protected static void prepareNextCertN(CertPath certPath, int index) {
      boolean[] var2;
      if ((var2 = ((X509Certificate)certPath.getCertificates().get(index)).getKeyUsage()) != null && !var2[5]) {
         throw new ExtCertPathValidatorException("Issuer certificate keyusage extension is critical and does not permit key signing.", (Throwable)null, certPath, index);
      }
   }

   protected static void prepareNextCertO(CertPath certPath, int index, Set criticalExtensions, List pathCheckers) {
      List var10000 = pathCheckers;
      X509Certificate pathCheckers1 = (X509Certificate)certPath.getCertificates().get(index);
      Iterator var4 = var10000.iterator();

      while(var4.hasNext()) {
         try {
            ((PKIXCertPathChecker)var4.next()).check(pathCheckers1, criticalExtensions);
         } catch (CertPathValidatorException var5) {
            String criticalExtensions1 = var5.getMessage();
            throw new CertPathValidatorException(criticalExtensions1, var5.getCause(), certPath, index);
         }
      }

      if (!criticalExtensions.isEmpty()) {
         throw new ExtCertPathValidatorException("Certificate has unsupported critical extension.", (Throwable)null, certPath, index);
      }
   }

   protected static int prepareNextCertH1(CertPath certPath, int index, int explicitPolicy) {
      return !CertPathValidatorUtilities.isSelfIssued((X509Certificate)certPath.getCertificates().get(index)) && explicitPolicy != 0 ? explicitPolicy - 1 : explicitPolicy;
   }

   protected static int prepareNextCertH2(CertPath certPath, int index, int policyMapping) {
      return !CertPathValidatorUtilities.isSelfIssued((X509Certificate)certPath.getCertificates().get(index)) && policyMapping != 0 ? policyMapping - 1 : policyMapping;
   }

   protected static int prepareNextCertH3(CertPath certPath, int index, int inhibitAnyPolicy) {
      return !CertPathValidatorUtilities.isSelfIssued((X509Certificate)certPath.getCertificates().get(index)) && inhibitAnyPolicy != 0 ? inhibitAnyPolicy - 1 : inhibitAnyPolicy;
   }

   protected static int wrapupCertA(int explicitPolicy, X509Certificate cert) {
      if (!CertPathValidatorUtilities.isSelfIssued(cert) && explicitPolicy != 0) {
         --explicitPolicy;
      }

      return explicitPolicy;
   }

   protected static int wrapupCertB(CertPath certPath, int index, int explicitPolicy) {
      X509Certificate var10000 = (X509Certificate)certPath.getCertificates().get(index);

      ASN1Sequence var8;
      try {
         var8 = ASN1Sequence.getInstance(CertPathValidatorUtilities.getExtensionValue(var10000, POLICY_CONSTRAINTS));
      } catch (AnnotatedException var6) {
         throw new ExtCertPathValidatorException("Policy constraints could no be decoded.", var6, certPath, index);
      }

      ASN1Sequence var3 = var8;
      if (var8 != null) {
         Enumeration var7 = var3.getObjects();

         while(var7.hasMoreElements()) {
            ASN1TaggedObject var4;
            if ((var4 = (ASN1TaggedObject)var7.nextElement()).getTagNo() == 0) {
               int var9;
               try {
                  var9 = DERInteger.getInstance(var4).getValue().intValue();
               } catch (Exception var5) {
                  throw new ExtCertPathValidatorException("Policy constraints requireExplicitPolicy field could no be decoded.", var5, certPath, index);
               }

               if (var9 == 0) {
                  return 0;
               }
            }
         }
      }

      return explicitPolicy;
   }

   protected static void wrapupCertF(CertPath certPath, int index, List pathCheckers, Set criticalExtensions) {
      List var10000 = pathCheckers;
      X509Certificate pathCheckers1 = (X509Certificate)certPath.getCertificates().get(index);
      Iterator var4 = var10000.iterator();

      while(var4.hasNext()) {
         try {
            ((PKIXCertPathChecker)var4.next()).check(pathCheckers1, criticalExtensions);
         } catch (CertPathValidatorException var5) {
            throw new ExtCertPathValidatorException("Additional certificate path checker failed.", var5, certPath, index);
         }
      }

      if (!criticalExtensions.isEmpty()) {
         throw new ExtCertPathValidatorException("Certificate has unsupported critical extension", (Throwable)null, certPath, index);
      }
   }

   protected static PKIXPolicyNode wrapupCertG(CertPath certPath, ExtendedPKIXParameters paramsPKIX, Set userInitialPolicySet, int index, List[] policyNodes, PKIXPolicyNode validPolicyTree, Set acceptablePolicies) {
      int var7 = certPath.getCertificates().size();
      if (validPolicyTree == null) {
         if (paramsPKIX.isExplicitPolicyRequired()) {
            throw new ExtCertPathValidatorException("Explicit policy requested but none available.", (Throwable)null, certPath, index);
         }

         certPath = null;
         validPolicyTree = certPath;
      } else {
         int certPath1;
         int paramsPKIX2;
         int userInitialPolicySet1;
         PKIXPolicyNode index1;
         Iterator index2;
         if (CertPathValidatorUtilities.isAnyPolicy(userInitialPolicySet)) {
            if (paramsPKIX.isExplicitPolicyRequired()) {
               if (acceptablePolicies.isEmpty()) {
                  throw new ExtCertPathValidatorException("Explicit policy requested but none available.", (Throwable)null, certPath, index);
               }

               HashSet paramsPKIX1;
               paramsPKIX1 = new HashSet.<init>();
               certPath1 = 0;

               for(userInitialPolicySet1 = certPath1; userInitialPolicySet1 < policyNodes.length; userInitialPolicySet1 = certPath1) {
                  List var8 = policyNodes[userInitialPolicySet1];

                  for(certPath1 = 0; certPath1 < var8.size(); ++certPath1) {
                     if ("2.5.29.32.0".equals((index1 = (PKIXPolicyNode)var8.get(certPath1)).getValidPolicy())) {
                        index2 = index1.getChildren();

                        while(index2.hasNext()) {
                           paramsPKIX1.add(index2.next());
                        }
                     }
                  }

                  certPath1 = userInitialPolicySet1 + 1;
               }

               Iterator certPath2 = paramsPKIX1.iterator();

               while(certPath2.hasNext()) {
                  acceptablePolicies.contains(((PKIXPolicyNode)certPath2.next()).getValidPolicy());
               }

               certPath1 = var7 - 1;

               for(paramsPKIX2 = certPath1; paramsPKIX2 >= 0; paramsPKIX2 = certPath1) {
                  List userInitialPolicySet2 = policyNodes[paramsPKIX2];

                  for(certPath1 = 0; certPath1 < userInitialPolicySet2.size(); ++certPath1) {
                     if (!(index1 = (PKIXPolicyNode)userInitialPolicySet2.get(certPath1)).hasChildren()) {
                        validPolicyTree = CertPathValidatorUtilities.removePolicyNode(validPolicyTree, policyNodes, index1);
                     }
                  }

                  certPath1 = paramsPKIX2 + -1;
               }
            }
         } else {
            HashSet certPath3;
            certPath3 = new HashSet.<init>();

            for(paramsPKIX2 = 0; paramsPKIX2 < policyNodes.length; ++paramsPKIX2) {
               List index3 = policyNodes[paramsPKIX2];

               for(int acceptablePolicies1 = 0; acceptablePolicies1 < index3.size(); ++acceptablePolicies1) {
                  PKIXPolicyNode var23;
                  if ("2.5.29.32.0".equals((var23 = (PKIXPolicyNode)index3.get(acceptablePolicies1)).getValidPolicy())) {
                     Iterator var24 = var23.getChildren();

                     while(var24.hasNext()) {
                        PKIXPolicyNode var9;
                        if (!"2.5.29.32.0".equals((var9 = (PKIXPolicyNode)var24.next()).getValidPolicy())) {
                           certPath3.add(var9);
                        }
                     }
                  }
               }
            }

            index2 = certPath3.iterator();

            while(index2.hasNext()) {
               PKIXPolicyNode certPath4;
               if (!userInitialPolicySet.contains((certPath4 = (PKIXPolicyNode)index2.next()).getValidPolicy())) {
                  validPolicyTree = CertPathValidatorUtilities.removePolicyNode(validPolicyTree, policyNodes, certPath4);
               }
            }

            if (validPolicyTree != null) {
               for(certPath1 = var7 - 1; certPath1 >= 0; --certPath1) {
                  List paramsPKIX3 = policyNodes[certPath1];

                  for(userInitialPolicySet1 = 0; userInitialPolicySet1 < paramsPKIX3.size(); ++userInitialPolicySet1) {
                     if (!(index1 = (PKIXPolicyNode)paramsPKIX3.get(userInitialPolicySet1)).hasChildren()) {
                        validPolicyTree = CertPathValidatorUtilities.removePolicyNode(validPolicyTree, policyNodes, index1);
                     }
                  }
               }
            }
         }
      }

      return validPolicyTree;
   }
}
